# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.compare_workflow_config_model_pair_maker_config import CompareWorkflowConfigModelPairMakerConfig
from rapidata.api_client.models.compare_workflow_model1_referee import CompareWorkflowModel1Referee
from rapidata.api_client.models.elo_config import EloConfig
from rapidata.api_client.models.feature_flag import FeatureFlag
from rapidata.api_client.models.file_asset_metadata_value import FileAssetMetadataValue
from typing import Optional, Set
from typing_extensions import Self

class CompareWorkflowConfigModel(BaseModel):
    """
    The configuration for creating a compare workflow. A compare workflow is a workflow that continuously matches datapoints against each other and updates their respective ELO scores. The ELO scores are used to determine the relative strength of the datapoints, and datapoints are matched against other datapoints with similar ELO scores. The end goal is a ranking of the datapoints based on their relative strength.
    """ # noqa: E501
    t: StrictStr = Field(description="Discriminator value for CompareWorkflowConfig", alias="_t")
    criteria: StrictStr = Field(description="The criteria to add to each compare rapid.")
    pair_maker_config: CompareWorkflowConfigModelPairMakerConfig = Field(alias="pairMakerConfig")
    referee: CompareWorkflowModel1Referee
    target_country_codes: List[StrictStr] = Field(description="A list of country codes that this workflow is targeting.", alias="targetCountryCodes")
    elo_config: Optional[EloConfig] = Field(default=None, alias="eloConfig")
    metadata: Dict[str, FileAssetMetadataValue] = Field(description="The metadata is attached to every single rapid and can be used for something like the prompt.")
    feature_flags: List[FeatureFlag] = Field(description="The list of feature flags that will be applied to the rapids created by this workflow.", alias="featureFlags")
    __properties: ClassVar[List[str]] = ["_t", "criteria", "pairMakerConfig", "referee", "targetCountryCodes", "eloConfig", "metadata", "featureFlags"]

    @field_validator('t')
    def t_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['CompareWorkflowConfig']):
            raise ValueError("must be one of enum values ('CompareWorkflowConfig')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CompareWorkflowConfigModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of pair_maker_config
        if self.pair_maker_config:
            _dict['pairMakerConfig'] = self.pair_maker_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of elo_config
        if self.elo_config:
            _dict['eloConfig'] = self.elo_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in metadata (dict)
        _field_dict = {}
        if self.metadata:
            for _key_metadata in self.metadata:
                if self.metadata[_key_metadata]:
                    _field_dict[_key_metadata] = self.metadata[_key_metadata].to_dict()
            _dict['metadata'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CompareWorkflowConfigModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_t": obj.get("_t") if obj.get("_t") is not None else 'CompareWorkflowConfig',
            "criteria": obj.get("criteria"),
            "pairMakerConfig": CompareWorkflowConfigModelPairMakerConfig.from_dict(obj["pairMakerConfig"]) if obj.get("pairMakerConfig") is not None else None,
            "referee": CompareWorkflowModel1Referee.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "targetCountryCodes": obj.get("targetCountryCodes"),
            "eloConfig": EloConfig.from_dict(obj["eloConfig"]) if obj.get("eloConfig") is not None else None,
            "metadata": dict(
                (_k, FileAssetMetadataValue.from_dict(_v))
                for _k, _v in obj["metadata"].items()
            )
            if obj.get("metadata") is not None
            else None,
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None
        })
        return _obj





from rapidata.api_client.models.add_validation_rapid_model import IRapidPayload
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_order_workflow_model_simple_workflow_model import (
    IOrderWorkflowModelSimpleWorkflowModel,
)
from rapidata.api_client.models.i_rapid_blueprint import IRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_line_rapid_blueprint import (
    IRapidBlueprintLineRapidBlueprint,
)
from rapidata.rapidata_client.workflow._base_workflow import Workflow
from rapidata.api_client.models.i_rapid_payload_line_payload import (
    IRapidPayloadLinePayload,
)
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.rapid_modality import RapidModality


class DrawWorkflow(Workflow):
    modality = RapidModality.LINE

    def __init__(self, target: str):
        super().__init__(type="SimpleWorkflowConfig")
        self._target = target

    def _get_instruction(self) -> str:
        return self._target

    def _to_model(self) -> IOrderWorkflowModel:
        blueprint = IRapidBlueprintLineRapidBlueprint(
            _t="LineBlueprint", target=self._target
        )

        return IOrderWorkflowModel(
            actual_instance=IOrderWorkflowModelSimpleWorkflowModel(
                _t="SimpleWorkflow",
                blueprint=IRapidBlueprint(actual_instance=blueprint),
            )
        )

    def _to_payload(self, datapoint: Datapoint) -> IRapidPayload:
        return IRapidPayload(
            actual_instance=IRapidPayloadLinePayload(
                _t="LinePayload",
                target=self._target,
            )
        )

    def __str__(self) -> str:
        return f"DrawWorkflow(target='{self._target}')"

    def __repr__(self) -> str:
        return f"DrawWorkflow(target={self._target!r})"



from abc import abstractmethod

from rapidata.api_client.models.i_user_filter_model import IUserFilterModel


class RapidataFilter:
    """The base class for all Rapidata Filters."""

    @abstractmethod
    def _to_model(self) -> IUserFilterModel:
        pass

    def __or__(self, other):
        """Enable the | operator to create OrFilter combinations."""
        if not isinstance(other, RapidataFilter):
            return NotImplemented

        from rapidata.rapidata_client.filter.or_filter import OrFilter

        # If self is already an OrFilter, extend its filters list
        if isinstance(self, OrFilter):
            if isinstance(other, OrFilter):
                return OrFilter(self.filters + other.filters)
            else:
                return OrFilter(self.filters + [other])
        # If other is an OrFilter, prepend self to its filters
        elif isinstance(other, OrFilter):
            return OrFilter([self] + other.filters)
        # Neither is an OrFilter, create a new one
        else:
            return OrFilter([self, other])

    def __and__(self, other):
        """Enable the & operator to create AndFilter combinations."""
        if not isinstance(other, RapidataFilter):
            return NotImplemented

        from rapidata.rapidata_client.filter.and_filter import AndFilter

        # If self is already an AndFilter, extend its filters list
        if isinstance(self, AndFilter):
            if isinstance(other, AndFilter):
                return AndFilter(self.filters + other.filters)
            else:
                return AndFilter(self.filters + [other])
        # If other is an AndFilter, prepend self to its filters
        elif isinstance(other, AndFilter):
            return AndFilter([self] + other.filters)
        # Neither is an AndFilter, create a new one
        else:
            return AndFilter([self, other])

    def __invert__(self):
        """Enable the ~ operator to create NotFilter negations."""
        from rapidata.rapidata_client.filter.not_filter import NotFilter

        # If self is already a NotFilter, return the original filter (double negation)
        if isinstance(self, NotFilter):
            return self.filter
        # Create a new NotFilter
        else:
            return NotFilter(filter=self)



from rapidata.api_client.models.prompt_asset_metadata_input import (
    PromptAssetMetadataInput,
)
from rapidata.rapidata_client.datapoints.metadata._base_metadata import Metadata
from rapidata.api_client.models.multi_asset_input_assets_inner import (
    ExistingAssetInput,
)
from rapidata.api_client.models.prompt_asset_metadata_input_asset import (
    PromptAssetMetadataInputAsset,
)
from pydantic import BaseModel


class MediaAssetMetadata(Metadata, BaseModel):
    internal_file_name: str

    def to_model(self):
        return PromptAssetMetadataInput(
            _t="PromptAssetMetadataInput",
            asset=PromptAssetMetadataInputAsset(
                actual_instance=ExistingAssetInput(
                    _t="ExistingAssetInput",
                    name=self.internal_file_name,
                ),
            ),
        )



from rapidata.api_client.models.add_validation_rapid_model import IRapidPayload
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_order_workflow_model_grouped_ranking_workflow_model import (
    IOrderWorkflowModelGroupedRankingWorkflowModel,
)
from rapidata.rapidata_client.workflow._base_workflow import Workflow
from rapidata.api_client.models.i_rapid_payload_compare_payload import (
    IRapidPayloadComparePayload,
)
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.rapid_modality import RapidModality
from rapidata.api_client.models.i_asset_input import IAssetInput


class MultiRankingWorkflow(Workflow):
    modality = RapidModality.COMPARE

    def __init__(
        self,
        instruction: str,
        comparison_budget_per_ranking: int,
        random_comparisons_ratio,
        elo_start: int = 1200,
        elo_k_factor: int = 40,
        elo_scaling_factor: int = 400,
        contexts: dict[str, str] | None = None,
        media_contexts: dict[str, IAssetInput] | None = None,
    ):
        from rapidata.api_client.models.i_pair_maker_config_model import (
            IPairMakerConfigModel,
        )
        from rapidata.api_client.models.i_pair_maker_config_model_online_pair_maker_config_model import (
            IPairMakerConfigModelOnlinePairMakerConfigModel,
        )
        from rapidata.api_client.models.elo_config_model import EloConfigModel

        super().__init__(type="CompareWorkflowConfig")

        self.contexts = contexts
        self.media_contexts = media_contexts

        self.instruction = instruction
        self.comparison_budget_per_ranking = comparison_budget_per_ranking
        self.random_comparisons_ratio = random_comparisons_ratio
        self.elo_start = elo_start
        self.elo_k_factor = elo_k_factor
        self.elo_scaling_factor = elo_scaling_factor

        self.pair_maker_config = IPairMakerConfigModel(
            actual_instance=IPairMakerConfigModelOnlinePairMakerConfigModel(
                _t="OnlinePairMaker",
                totalComparisonBudget=comparison_budget_per_ranking,
                randomMatchesRatio=random_comparisons_ratio,
            ),
        )

        self.elo_config = EloConfigModel(
            startingElo=elo_start,
            kFactor=elo_k_factor,
            scalingFactor=elo_scaling_factor,
        )

    def _to_model(self) -> IOrderWorkflowModel:
        return IOrderWorkflowModel(
            actual_instance=IOrderWorkflowModelGroupedRankingWorkflowModel(
                _t="GroupedRankingWorkflow",
                criteria=self.instruction,
                eloConfig=self.elo_config,
                pairMakerConfig=self.pair_maker_config,
                contexts=self.contexts,
                contextAssets=self.media_contexts,
            )
        )

    def _get_instruction(self) -> str:
        return self.instruction

    def _to_payload(self, datapoint: Datapoint) -> IRapidPayload:
        return IRapidPayload(
            actual_instance=IRapidPayloadComparePayload(
                _t="ComparePayload",
                criteria=self.instruction,
            )
        )

    def __str__(self) -> str:
        return f"MultiRankingWorkflow(instruction='{self.instruction}')"

    def __repr__(self) -> str:
        return f"MultiRankingWorkflow(instruction={self.instruction!r}, comparison_budget_per_ranking={self.comparison_budget_per_ranking!r}, random_comparisons_ratio={self.random_comparisons_ratio!r}, elo_start={self.elo_start!r}, elo_k_factor={self.elo_k_factor!r}, elo_scaling_factor={self.elo_scaling_factor!r})"



from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.service.openapi_service import OpenAPIService
from concurrent.futures import ThreadPoolExecutor, as_completed
from tqdm import tqdm

from rapidata.rapidata_client.config import logger
import time
from rapidata.rapidata_client.api.rapidata_api_client import (
    suppress_rapidata_error_logging,
)
from rapidata.rapidata_client.config.rapidata_config import rapidata_config
from rapidata.rapidata_client.datapoints._datapoint_uploader import DatapointUploader

# Add OpenTelemetry context imports for thread propagation
from opentelemetry import context as otel_context


class RapidataDataset:
    def __init__(self, dataset_id: str, openapi_service: OpenAPIService):
        self.id = dataset_id
        self.openapi_service = openapi_service
        self.datapoint_uploader = DatapointUploader(openapi_service)

    def add_datapoints(
        self,
        datapoints: list[Datapoint],
    ) -> tuple[list[Datapoint], list[Datapoint]]:
        """
        Process uploads concurrently with a ThreadPoolExecutor.

        Args:
            datapoints: List of datapoints to upload

        Returns:
            tuple[list[Datapoint], list[Datapoint]]: Lists of successful and failed uploads
        """
        successful_uploads: list[Datapoint] = []
        failed_uploads: list[Datapoint] = []

        with tqdm(
            total=len(datapoints),
            desc="Uploading datapoints",
            disable=rapidata_config.logging.silent_mode,
        ) as progress_bar:

            def process_upload_with_context(
                context: otel_context.Context, datapoint: Datapoint, index: int
            ) -> tuple[list[Datapoint], list[Datapoint]]:
                """Wrapper function that runs _process_single_upload with the provided context."""
                token = otel_context.attach(context)
                try:
                    return self._process_single_upload(datapoint, index)
                finally:
                    otel_context.detach(token)

            # Capture the current OpenTelemetry context before creating threads
            current_context = otel_context.get_current()

            with ThreadPoolExecutor(
                max_workers=rapidata_config.upload.maxWorkers
            ) as executor:
                # Submit all uploads at once - executor manages concurrency via max_workers
                futures = [
                    executor.submit(
                        process_upload_with_context,
                        current_context,
                        datapoint,
                        i,
                    )
                    for i, datapoint in enumerate(datapoints)
                ]

                # Process results as they complete (no blocking on chunks)
                for future in as_completed(futures):
                    try:
                        upload_successful, upload_failed = future.result()
                        successful_uploads.extend(upload_successful)
                        failed_uploads.extend(upload_failed)
                        progress_bar.update(len(upload_successful))
                    except Exception as e:
                        logger.error("Future execution failed: %s", str(e))

        if failed_uploads:
            logger.error(
                "Upload failed for %s datapoints: %s",
                len(failed_uploads),
                failed_uploads,
            )

        return successful_uploads, failed_uploads

    def _process_single_upload(
        self,
        datapoint: Datapoint,
        index: int,
    ) -> tuple[list[Datapoint], list[Datapoint]]:
        """
        Process single upload with retry logic and error tracking.

        Args:
            media_asset: MediaAsset or MultiAsset to upload
            meta_list: Optional sequence of metadata for the asset
            index: Sort index for the upload
            max_retries: Maximum number of retry attempts (default: 3)

        Returns:
            tuple[list[Datapoint], list[Datapoint]]: Lists of successful and failed datapoints
        """
        logger.debug("Processing single upload for %s with index %s", datapoint, index)

        local_successful: list[Datapoint] = []
        local_failed: list[Datapoint] = []

        last_exception = None
        for attempt in range(rapidata_config.upload.maxRetries):
            try:
                with suppress_rapidata_error_logging():
                    self.datapoint_uploader.upload_datapoint(
                        dataset_id=self.id,
                        datapoint=datapoint,
                        index=index,
                    )

                local_successful.append(datapoint)

                return local_successful, local_failed

            except Exception as e:
                last_exception = e
                if attempt < rapidata_config.upload.maxRetries - 1:
                    # Exponential backoff: wait 1s, then 2s, then 4s
                    retry_delay = 2**attempt
                    time.sleep(retry_delay)
                    logger.debug("Error: %s", str(last_exception))
                    logger.debug(
                        "Retrying %s of %s...",
                        attempt + 1,
                        rapidata_config.upload.maxRetries,
                    )

        # If we get here, all retries failed
        local_failed.append(datapoint)
        tqdm.write(
            f"Upload failed for {datapoint} after {rapidata_config.upload.maxRetries} attempts. \nFinal error: \n{str(last_exception)}"
        )

        return local_successful, local_failed

    def __str__(self) -> str:
        return f"RapidataDataset(id={self.id})"

    def __repr__(self) -> str:
        return self.__str__()



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.bounding_box_payload import BoundingBoxPayload
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.compare_payload import ComparePayload
from rapidata.api_client.models.free_text_payload import FreeTextPayload
from rapidata.api_client.models.line_payload import LinePayload
from rapidata.api_client.models.locate_payload import LocatePayload
from rapidata.api_client.models.named_entity_payload import NamedEntityPayload
from rapidata.api_client.models.polygon_payload import PolygonPayload
from rapidata.api_client.models.scrub_payload import ScrubPayload
from rapidata.api_client.models.transcription_payload import TranscriptionPayload
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

VALIDATIONSETVALIDATIONSETIDRAPIDPOSTPAYLOADPARAMETER_ONE_OF_SCHEMAS = ["BoundingBoxPayload", "ClassifyPayload", "ComparePayload", "FreeTextPayload", "LinePayload", "LocatePayload", "NamedEntityPayload", "PolygonPayload", "ScrubPayload", "TranscriptionPayload"]

class ValidationSetValidationSetIdRapidPostPayloadParameter(BaseModel):
    """
    ValidationSetValidationSetIdRapidPostPayloadParameter
    """
    # data type: TranscriptionPayload
    oneof_schema_1_validator: Optional[TranscriptionPayload] = None
    # data type: ScrubPayload
    oneof_schema_2_validator: Optional[ScrubPayload] = None
    # data type: PolygonPayload
    oneof_schema_3_validator: Optional[PolygonPayload] = None
    # data type: NamedEntityPayload
    oneof_schema_4_validator: Optional[NamedEntityPayload] = None
    # data type: LocatePayload
    oneof_schema_5_validator: Optional[LocatePayload] = None
    # data type: LinePayload
    oneof_schema_6_validator: Optional[LinePayload] = None
    # data type: FreeTextPayload
    oneof_schema_7_validator: Optional[FreeTextPayload] = None
    # data type: ComparePayload
    oneof_schema_8_validator: Optional[ComparePayload] = None
    # data type: ClassifyPayload
    oneof_schema_9_validator: Optional[ClassifyPayload] = None
    # data type: BoundingBoxPayload
    oneof_schema_10_validator: Optional[BoundingBoxPayload] = None
    actual_instance: Optional[Union[BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload]] = None
    one_of_schemas: Set[str] = { "BoundingBoxPayload", "ClassifyPayload", "ComparePayload", "FreeTextPayload", "LinePayload", "LocatePayload", "NamedEntityPayload", "PolygonPayload", "ScrubPayload", "TranscriptionPayload" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = ValidationSetValidationSetIdRapidPostPayloadParameter.model_construct()
        error_messages = []
        match = 0
        # validate data type: TranscriptionPayload
        if not isinstance(v, TranscriptionPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TranscriptionPayload`")
        else:
            match += 1
        # validate data type: ScrubPayload
        if not isinstance(v, ScrubPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ScrubPayload`")
        else:
            match += 1
        # validate data type: PolygonPayload
        if not isinstance(v, PolygonPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PolygonPayload`")
        else:
            match += 1
        # validate data type: NamedEntityPayload
        if not isinstance(v, NamedEntityPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `NamedEntityPayload`")
        else:
            match += 1
        # validate data type: LocatePayload
        if not isinstance(v, LocatePayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocatePayload`")
        else:
            match += 1
        # validate data type: LinePayload
        if not isinstance(v, LinePayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LinePayload`")
        else:
            match += 1
        # validate data type: FreeTextPayload
        if not isinstance(v, FreeTextPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `FreeTextPayload`")
        else:
            match += 1
        # validate data type: ComparePayload
        if not isinstance(v, ComparePayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ComparePayload`")
        else:
            match += 1
        # validate data type: ClassifyPayload
        if not isinstance(v, ClassifyPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ClassifyPayload`")
        else:
            match += 1
        # validate data type: BoundingBoxPayload
        if not isinstance(v, BoundingBoxPayload):
            error_messages.append(f"Error! Input type `{type(v)}` is not `BoundingBoxPayload`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in ValidationSetValidationSetIdRapidPostPayloadParameter with oneOf schemas: BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in ValidationSetValidationSetIdRapidPostPayloadParameter with oneOf schemas: BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into TranscriptionPayload
        try:
            instance.actual_instance = TranscriptionPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ScrubPayload
        try:
            instance.actual_instance = ScrubPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PolygonPayload
        try:
            instance.actual_instance = PolygonPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into NamedEntityPayload
        try:
            instance.actual_instance = NamedEntityPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocatePayload
        try:
            instance.actual_instance = LocatePayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LinePayload
        try:
            instance.actual_instance = LinePayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into FreeTextPayload
        try:
            instance.actual_instance = FreeTextPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ComparePayload
        try:
            instance.actual_instance = ComparePayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ClassifyPayload
        try:
            instance.actual_instance = ClassifyPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into BoundingBoxPayload
        try:
            instance.actual_instance = BoundingBoxPayload.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into ValidationSetValidationSetIdRapidPostPayloadParameter with oneOf schemas: BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into ValidationSetValidationSetIdRapidPostPayloadParameter with oneOf schemas: BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], BoundingBoxPayload, ClassifyPayload, ComparePayload, FreeTextPayload, LinePayload, LocatePayload, NamedEntityPayload, PolygonPayload, ScrubPayload, TranscriptionPayload]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictBytes, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Tuple, Union
from rapidata.api_client.models.entity_tag_header_value import EntityTagHeaderValue
from typing import Optional, Set
from typing_extensions import Self

class FileStreamResult(BaseModel):
    """
    FileStreamResult
    """ # noqa: E501
    file_stream: Optional[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]] = Field(default=None, alias="fileStream")
    content_type: Optional[StrictStr] = Field(default=None, alias="contentType")
    file_download_name: Optional[StrictStr] = Field(default=None, alias="fileDownloadName")
    last_modified: Optional[datetime] = Field(default=None, alias="lastModified")
    entity_tag: Optional[EntityTagHeaderValue] = Field(default=None, alias="entityTag")
    enable_range_processing: Optional[StrictBool] = Field(default=None, alias="enableRangeProcessing")
    __properties: ClassVar[List[str]] = ["fileStream", "contentType", "fileDownloadName", "lastModified", "entityTag", "enableRangeProcessing"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FileStreamResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of entity_tag
        if self.entity_tag:
            _dict['entityTag'] = self.entity_tag.to_dict()
        # set to None if content_type (nullable) is None
        # and model_fields_set contains the field
        if self.content_type is None and "content_type" in self.model_fields_set:
            _dict['contentType'] = None

        # set to None if file_download_name (nullable) is None
        # and model_fields_set contains the field
        if self.file_download_name is None and "file_download_name" in self.model_fields_set:
            _dict['fileDownloadName'] = None

        # set to None if last_modified (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified is None and "last_modified" in self.model_fields_set:
            _dict['lastModified'] = None

        # set to None if entity_tag (nullable) is None
        # and model_fields_set contains the field
        if self.entity_tag is None and "entity_tag" in self.model_fields_set:
            _dict['entityTag'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FileStreamResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "fileStream": obj.get("fileStream"),
            "contentType": obj.get("contentType"),
            "fileDownloadName": obj.get("fileDownloadName"),
            "lastModified": obj.get("lastModified"),
            "entityTag": EntityTagHeaderValue.from_dict(obj["entityTag"]) if obj.get("entityTag") is not None else None,
            "enableRangeProcessing": obj.get("enableRangeProcessing")
        })
        return _obj





from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting
from rapidata.rapidata_client.config import managed_print


class FreeTextMinimumCharacters(RapidataSetting):
    """
    Set the minimum number of characters a user has to type.

    Args:
        value (int): The minimum number of characters for free text.
    """

    def __init__(self, value: int):
        if value < 1:
            raise ValueError(
                "The minimum number of characters must be greater than or equal to 1."
            )
        if value > 40:
            managed_print(
                f"Warning: Are you sure you want to set the minimum number of characters at {value}?"
            )
        super().__init__(key="free_text_minimum_characters", value=value)



from __future__ import annotations

from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from pydantic import BaseModel


class CustomFilter(RapidataFilter, BaseModel):
    """CustomFilter Class

    Can be used to filter who to target based on custom filters.

    Ought to be used with contact to Rapidata.

    Warning: If identifier does not exist, order will not get any responses.

    Args:
        identifier (str): Identifier of the custom filter.
        values (list[str]): List of values to filter by.
    """

    identifier: str
    values: list[str]

    def __init__(self, identifier: str, values: list[str]):
        super().__init__(identifier=identifier, values=values)

    def _to_model(self):
        from rapidata.api_client.models.i_user_filter_model import IUserFilterModel
        from rapidata.api_client.models.i_user_filter_model_custom_user_filter_model import (
            IUserFilterModelCustomUserFilterModel,
        )

        return IUserFilterModel(
            actual_instance=IUserFilterModelCustomUserFilterModel(
                _t="CustomFilter",
                identifier=self.identifier,
                values=self.values,
            )
        )



# flake8: noqa

# import apis into api package
from rapidata.api_client.api.asset_api import AssetApi
from rapidata.api_client.api.audience_api import AudienceApi
from rapidata.api_client.api.benchmark_api import BenchmarkApi
from rapidata.api_client.api.campaign_api import CampaignApi
from rapidata.api_client.api.client_api import ClientApi
from rapidata.api_client.api.compare_workflow_api import CompareWorkflowApi
from rapidata.api_client.api.customer_rapid_api import CustomerRapidApi
from rapidata.api_client.api.datapoint_api import DatapointApi
from rapidata.api_client.api.dataset_api import DatasetApi
from rapidata.api_client.api.evaluation_workflow_api import EvaluationWorkflowApi
from rapidata.api_client.api.feedback_api import FeedbackApi
from rapidata.api_client.api.grouped_ranking_workflow_api import GroupedRankingWorkflowApi
from rapidata.api_client.api.identity_api import IdentityApi
from rapidata.api_client.api.leaderboard_api import LeaderboardApi
from rapidata.api_client.api.newsletter_api import NewsletterApi
from rapidata.api_client.api.order_api import OrderApi
from rapidata.api_client.api.participant_api import ParticipantApi
from rapidata.api_client.api.pipeline_api import PipelineApi
from rapidata.api_client.api.prompt_api import PromptApi
from rapidata.api_client.api.rapidata_identity_api_api import RapidataIdentityAPIApi
from rapidata.api_client.api.sample_api import SampleApi
from rapidata.api_client.api.simple_workflow_api import SimpleWorkflowApi
from rapidata.api_client.api.survey_api import SurveyApi
from rapidata.api_client.api.user_rapid_api import UserRapidApi
from rapidata.api_client.api.validation_set_api import ValidationSetApi
from rapidata.api_client.api.workflow_api import WorkflowApi




from __future__ import annotations

from typing import Sequence

from rapidata.rapidata_client.selection._base_selection import RapidataSelection


class ShufflingSelection(RapidataSelection):
    """ShufflingSelection Class

    Shuffles the selections provided in the list.

    Args:
        selections (Sequence[RapidataSelection]): List of selections to shuffle.

    Example:
        ```python
        selection = ShufflingSelection(
                    [ValidSelections("validation_id", 1), LabelingSelection(2)])
        ```
        This means that the users will get 1 validation task and 2 labeling tasks in a shuffled order.
    """

    def __init__(self, selections: Sequence[RapidataSelection]):
        self.selections = selections

    def _to_model(self):
        from rapidata.api_client.models.i_selection import ISelection
        from rapidata.api_client.models.i_selection_shuffling_selection import (
            ISelectionShufflingSelection,
        )

        return ISelection(
            actual_instance=ISelectionShufflingSelection(
                _t="ShufflingSelection",
                selections=[selection._to_model() for selection in self.selections],
            )
        )

    def __str__(self) -> str:
        return f"ShufflingSelection(selections={self.selections})"

    def __repr__(self) -> str:
        return self.__str__()



from rapidata.rapidata_client.datapoints.metadata._base_metadata import Metadata
from rapidata.api_client.models.private_text_metadata_input import (
    PrivateTextMetadataInput,
)
from pydantic import BaseModel


class PromptIdentifierMetadata(Metadata, BaseModel):
    identifier: str

    def to_model(self):
        return PrivateTextMetadataInput(
            _t="PrivateTextMetadataInput", identifier="prompt-id", text=self.identifier
        )



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.elo_config import EloConfig
from rapidata.api_client.models.i_asset_model import IAssetModel
from rapidata.api_client.models.i_pair_maker_information import IPairMakerInformation
from rapidata.api_client.models.i_referee_config import IRefereeConfig
from typing import Optional, Set
from typing_extensions import Self

class IWorkflowModelCompareWorkflowModel(BaseModel):
    """
    IWorkflowModelCompareWorkflowModel
    """ # noqa: E501
    t: StrictStr = Field(alias="_t")
    id: StrictStr
    referee: IRefereeConfig
    pair_maker_information: IPairMakerInformation = Field(alias="pairMakerInformation")
    state: StrictStr
    criteria: StrictStr
    name: StrictStr
    elo_config: EloConfig = Field(alias="eloConfig")
    context: Optional[StrictStr]
    context_asset: Optional[IAssetModel] = Field(alias="contextAsset")
    owner_mail: Optional[StrictStr] = Field(alias="ownerMail")
    __properties: ClassVar[List[str]] = ["_t", "id", "referee", "pairMakerInformation", "state", "criteria", "name", "eloConfig", "context", "contextAsset", "ownerMail"]

    @field_validator('t')
    def t_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['CompareWorkflowModel']):
            raise ValueError("must be one of enum values ('CompareWorkflowModel')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of IWorkflowModelCompareWorkflowModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pair_maker_information
        if self.pair_maker_information:
            _dict['pairMakerInformation'] = self.pair_maker_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of elo_config
        if self.elo_config:
            _dict['eloConfig'] = self.elo_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of context_asset
        if self.context_asset:
            _dict['contextAsset'] = self.context_asset.to_dict()
        # set to None if context (nullable) is None
        # and model_fields_set contains the field
        if self.context is None and "context" in self.model_fields_set:
            _dict['context'] = None

        # set to None if context_asset (nullable) is None
        # and model_fields_set contains the field
        if self.context_asset is None and "context_asset" in self.model_fields_set:
            _dict['contextAsset'] = None

        # set to None if owner_mail (nullable) is None
        # and model_fields_set contains the field
        if self.owner_mail is None and "owner_mail" in self.model_fields_set:
            _dict['ownerMail'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of IWorkflowModelCompareWorkflowModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_t": obj.get("_t"),
            "id": obj.get("id"),
            "referee": IRefereeConfig.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "pairMakerInformation": IPairMakerInformation.from_dict(obj["pairMakerInformation"]) if obj.get("pairMakerInformation") is not None else None,
            "state": obj.get("state"),
            "criteria": obj.get("criteria"),
            "name": obj.get("name"),
            "eloConfig": EloConfig.from_dict(obj["eloConfig"]) if obj.get("eloConfig") is not None else None,
            "context": obj.get("context"),
            "contextAsset": IAssetModel.from_dict(obj["contextAsset"]) if obj.get("contextAsset") is not None else None,
            "ownerMail": obj.get("ownerMail")
        })
        return _obj


